/*
 * Copyright © 2022 DATAMART LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ru.datamart.kafka.clickhouse.reader.configuration;

import com.clickhouse.client.config.ClickHouseSslMode;
import io.vertx.core.Vertx;
import io.vertx.ext.jdbc.JDBCClient;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import lombok.val;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import ru.datamart.kafka.clickhouse.reader.configuration.properties.DatasourceProperties;
import ru.datamart.kafka.clickhouse.reader.converter.ClickhouseFromSqlTypeConverter;
import ru.datamart.kafka.clickhouse.reader.service.ClickhouseExecutor;
import ru.yandex.clickhouse.settings.ClickHouseProperties;

import java.util.Arrays;
import java.util.List;
import java.util.Properties;
import java.util.stream.Collectors;

@Configuration
@Slf4j
public class QueryConfiguration {

    @Bean("clickhouseDatabaseExecutorList")
    public List<ClickhouseExecutor> clickhouseExecutors(Vertx vertx,
                                                        DatasourceProperties clickhouseProperties,
                                                        ClickhouseFromSqlTypeConverter converter) {
        val hostList = getHostList(clickhouseProperties.getHosts());
        List<ClickhouseExecutor> databaseExecutors = hostList.stream()
                .map(host -> {
                    val url = String.format("jdbc:clickhouse://%s/%s", host.trim(),
                            clickhouseProperties.getDatabase());
                    val properties = new ClickHouseProperties();
                    properties.setUser(clickhouseProperties.getUser());
                    properties.setPassword(clickhouseProperties.getPassword());
                    properties.setEnableOptimizePredicateExpression(false);
                    properties.setSslMode(ClickHouseSslMode.NONE.name());
                    val dataSource = createDatasource(url, properties.asProperties());
                    val client = JDBCClient.create(vertx, dataSource);
                    return new ClickhouseExecutor(client, converter);
                })
                .collect(Collectors.toList());
        log.debug("Created clickhouse executors {}", hostList);
        return databaseExecutors;
    }

    @SneakyThrows
    private ClickhouseDataSource createDatasource(String url, Properties properties) {
        return new ClickhouseDataSource(url, properties);
    }

    private List<String> getHostList(String hosts) {
        return Arrays.stream(hosts.trim().split(",")).collect(Collectors.toList());
    }

}
