/*
 * Copyright © 2022 DATAMART LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ru.datamart.kafka.clickhouse.writer.dao.impl;

import io.vertx.core.AsyncResult;
import io.vertx.core.Future;
import io.vertx.core.Handler;
import io.vertx.ext.sql.SQLClient;
import lombok.extern.slf4j.Slf4j;
import lombok.val;
import ru.datamart.kafka.clickhouse.writer.dao.ClickhouseDao;
import ru.datamart.kafka.clickhouse.writer.model.sql.ClickhouseInsertSqlRequest;
import ru.datamart.kafka.clickhouse.writer.model.sql.InsertSqlRequest;

@Slf4j
public class ClickhouseDaoImpl implements ClickhouseDao {
    private final SQLClient sqlClient;

    public ClickhouseDaoImpl(SQLClient sqlClient) {
        this.sqlClient = sqlClient;
    }

    @Override
    public void insertRows(InsertSqlRequest request, Handler<AsyncResult<Integer>> handler) {
        sqlClient.getConnection(ar -> {
            if (ar.failed()) {
                log.error("Connection error: " + ar.cause().getMessage());
                handler.handle(Future.failedFuture(ar.cause()));
                return;
            }

            val conn = ar.result();
            val params = ((ClickhouseInsertSqlRequest) request).getParams();
            conn.batchWithParams(request.getSql(), params, batchAr -> {
                conn.close();

                if (batchAr.failed()) {
                    handler.handle(Future.failedFuture(batchAr.cause()));
                    return;
                }

                log.debug("Insert rows query executed successfully: [{}]", request.getSql());
                val result = batchAr.result();
                handler.handle(Future.succeededFuture(result.size()));
            });
        });
    }
}
