/*
 * Copyright © 2022 DATAMART LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ru.datamart.kafka.clickhouse.writer.model;

import lombok.Data;

import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicInteger;

@Data
public class InsertDataContext {
    private final InsertDataContextId id;
    private final InsertDataRequest request;
    private final AtomicInteger activeChunks = new AtomicInteger(0);
    private final List<String> verticleIds = new CopyOnWriteArrayList<>();
    private final AtomicBoolean started = new AtomicBoolean(false);
    private final long startTime = System.currentTimeMillis();
    private volatile String insertSql;
    private volatile Throwable cause;

    public InsertDataContext(InsertDataRequest request, InsertDataContextId id) {
        this.request = request;
        this.id = id;
    }

    public long getProcessingTime() {
        return System.currentTimeMillis() - startTime;
    }
}
