/*
 * Copyright © 2022 DATAMART LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ru.datamart.kafka.clickhouse.writer.model;

import lombok.Data;
import lombok.EqualsAndHashCode;

import java.util.Locale;

@Data
@EqualsAndHashCode
public class InsertDataContextId {
    private static final String ID_TEMPLATE = "%s_%s_%s";

    private final String id;

    public InsertDataContextId(String id) {
        this.id = id.toLowerCase(Locale.ROOT);
    }

    public static InsertDataContextId of(InsertDataRequest insertDataRequest) {
        return new InsertDataContextId(String.format(ID_TEMPLATE, insertDataRequest.getDatamart(), insertDataRequest.getTableName(), insertDataRequest.getKafkaTopic()));
    }

    public static InsertDataContextId of(StopRequest stopRequest) {
        return new InsertDataContextId(String.format(ID_TEMPLATE, stopRequest.getDatamart(), stopRequest.getTableName(), stopRequest.getKafkaTopic()));
    }

    public static InsertDataContextId of(String id) {
        return new InsertDataContextId(id);
    }

    @Override
    public String toString() {
        return id;
    }
}
